/***************************************************************************//**
*   @file   main.c
*   @brief  AD7091R Driver.
*   @author ATofan (alexandru.tofan@analog.com)
********************************************************************************
* Copyright 2012(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
********************************************************************************
*   SVN Revision: $WCREV$
*******************************************************************************/

#include <stdio.h>
#include "xparameters.h"
#include "xil_cache.h"
#include "xil_io.h"
#define AD7091_BASEADDR XPAR_AD7091R_0_BASEADDR

void printVin(void);
float setVref(void);


const float AVDD = 5.25;
float       vref = 2.5;

int main() 
{
   Xil_ICacheEnable();
   Xil_DCacheEnable();

   vref = setVref();

   while(1)
   {
	   // Print value read from ad7091r
	   printVin();
	   // wait for key press
	   while(!((Xil_In32(XPAR_RS232_UART_1_BASEADDR + 0x08)) & 0x01));
	   Xil_In32(XPAR_RS232_UART_1_BASEADDR);
   }

   Xil_DCacheDisable();
   Xil_ICacheDisable();

   return 0;
}

/******************************************************************************
* @brief reads data from ad7091r and prints it via uart.
******************************************************************************/
void printVin(void)
{
	int 	data 		= 0;
	float 	value 		= 0;
	int 	whole 		= 0;
	int 	thousands 	= 0;

 	data = Xil_In32(AD7091_BASEADDR);
 	// wait for conversion to complete
   	while((data&0x10000)!=0x10000)
   	{
   		data = Xil_In32(AD7091_BASEADDR);
   	}
   	data = data & 0xFFF;

    // converting data for display
   	value = ((float)data / 4096) * vref;
   	whole = (int)value;
   	thousands = ((value - whole) * 1000);

	if(thousands > 99)
	{
		xil_printf("Vin = %d.%3d %c\n\r",whole, thousands,'V');
	}
	else if(thousands > 9)
	{
		xil_printf("Vin = %d.0%2d %c\n\r",whole, thousands,'V');
	}
	else
	{
		xil_printf("Vin = %d.00%1d %c\n\r",whole, thousands,'V');
	}
    xil_printf("\n\rPress any key for a new conversion\n\r");
}

/******************************************************************************
* @brief sets VREF value and prints it via uart.
******************************************************************************/
float setVref(void)
{
	char 	rx      = 0;
	char 	c[4]    = "0000";
	char 	*c_ptr;
	int 	i       = 0;
	char 	cnt     = 0;
	char 	valid   = 0;
	float 	value   = 0;
	float 	avdd_mv = 0;


	// Disable interrupts in order to use normal polling
	microblaze_disable_interrupts();

	avdd_mv = AVDD * 1000;

	xil_printf("\n\rVREF is set to 2500mV.");
	xil_printf("\n\rTo continue press [Enter], to change VREF press another key.");

	// Check if data is available on UART
	while(!((Xil_In32(XPAR_RS232_UART_1_BASEADDR + 0x08)) & 0x01));
	// Store and display received data
	rx = Xil_In32(XPAR_RS232_UART_1_BASEADDR);
	// Check if pressed key is [Enter]
	if((rx == 0x0D) || (rx == 0x0A))
	{
		value = 2500;
	}
	else
	{
	// Maximum number of characters will be 6 (e.g. 3 3 0 0 0x0D 0x0A)
	while(valid != 1)
	{
		cnt = 0;
		i = 0;
		value = 0;
		c_ptr = c;

		xil_printf("\n\rPlease enter a value between 2700 and AVDD [mV]: ");
		while(i < 6)
		{
			// Check if data is available on UART
			while(!((Xil_In32(XPAR_RS232_UART_1_BASEADDR + 0x08)) & 0x01));
			// Store and display received data
			rx = Xil_In32(XPAR_RS232_UART_1_BASEADDR);
			xil_printf("%c", rx);
			// Check if pressed key is [Enter]
			if(rx == 0x0D)
			{
				i = 5;
			}
			else if(rx == 0x0A)
			{
				i = 5;
			}
			// If entered key is not [0] to [9]
			else if((rx < 0x30) || (rx > 0x39))
			{
				xil_printf("\n\rCharacters entered must be from [0] to [9]!\n\r");
				i = 6;
				valid = 0;
			}
			// If everything is ok, save data
			else
			{
				*c_ptr++ = rx;
				cnt = cnt + 1;
				valid = 1;
			}
			// If 4 characteres have been entered, stop.
			if(cnt == 4)
			{
				i = 6;
			}
			i++;
		}
		// Check if number is bigger than 5000

		// Translate from ASCII to decimal
		for(i = 0; i < cnt; i++)
		{
			value = value * 10 + (c[i] - 0x30);
		}
		if (((value > avdd_mv) || (value < 2700)) && (valid == 1))
		{
			xil_printf("\n\rValue entered must be between 2700 and AVDD!\n\r");
			valid = 0;
		}
	}
	}

	xil_printf("\n\r\n\r>Reference value is: ");
	xil_printf("%d", (int)value);
	xil_printf(" mV\n\r\n\r");

	// Enable interrupts
	microblaze_enable_interrupts();

	// Overwrite current value of vref
	vref = value / 1000;
	return vref;

}

